# Twig Placeholders

## Overview

Twig Placeholders provides Twig functions for generating placeholder content in your templates. This includes dynamically generated Lorem Ipsum text and placeholder images, making it useful for prototyping, development, and theming and is especially powerful when demonstrating Drupal single directory components in component libraries.

## Features

- **Generate Lorem Ipsum text** using a simple Twig function.
- **Generate placeholder images** from [Lorem Picsum](https://picsum.photos/) with customisable attributes.
- **Generate placeholder videos** from [Test Videos](https://test-videos.co.uk) with customisable attributes.
- **Fully configurable** via Twig function arguments.
- **Works seamlessly** with Drupal 10.3 and above.

## Usage

### Generating Lorem Ipsum Text

The `tp_lorem_ipsum` function generates Lorem Ipsum text with a specified word count.

```twig
<p>{{ tp_lorem_ipsum(15) }}</p>
```

**Arguments:**
- **(optional) `word_count` (int):** Number of words to generate. Defaults to **15** if omitted.

### Generating Placeholder Images

The `tp_image` function generates a placeholder image URL or an entire `<img>` tag with customisable attributes.

```twig
{{ tp_image() }}
```

#### **Optional Arguments:**
| Parameter   | Type   | Default | Description |
|-------------|--------|---------|-------------|
| `url_only`  | bool   | `false` | If `true`, returns only the image URL instead of an `<img>` tag. |
| `width`     | int    | `800`   | Width of the image (px). |
| `height`    | int    | `450`   | Height of the image (px). |
| `image_id`  | int    | `null`  | Specific image ID from Picsum Photos. See [Lorem Picsum IDs](https://picsum.photos/images) |
| `grayscale` | bool   | `false` | Apply grayscale filter if `true`. |
| `blur`      | int    | `null`  | Blur level (1-10). |
| `extension` | string | `null`  | Image format (`webp` or `jpg`). |

#### **Examples:**

**Generate a default placeholder image:**
```twig
{{ tp_image() }}
```
_Output:_
```html
<img src="https://picsum.photos/800/450" alt="Placeholder image" width="800" height="450">
```

**Generate only the image URL:**
```twig
{{ tp_image(true) }}
```
_Output:_
```text
https://picsum.photos/800/450
```

**Generate a grayscale image:**
```twig
{{ tp_image(false, 600, 400, null, true) }}
```
_Output:_
```html
<img src="https://picsum.photos/600/400?grayscale" alt="Placeholder image" width="600" height="400">
```

**Generate a blurred image (level 5):**
```twig
{{ tp_image(false, 500, 300, null, false, 5) }}
```
_Output:_
```html
<img src="https://picsum.photos/500/300?blur=5" alt="Placeholder image" width="500" height="300">
```

### Generating Placeholder Select Data
The `tp_select_data` function generates a list of `<option>` placeholder data with optional `<optgroup>` wrappers and the ability to equally distribute the options among the optgroups or randomly place them instead.

```twig
{{ tp_select_data() }}
```

#### **Optional Arguments:**
| Parameter       | Type | Default | Description |
|-----------------|------|---------|-------------|
| `num_items`     | int  | `10`    | The total number of options to return. |
| `num_optgroups` | int  | `0`     | The total number of optgroups to return. |
| `randomise`     | bol  | `false` | If `true` options will be distributed randomly across optgroups. If `false` options will be distributed equally among optgroups. |

#### **Examples:**

**Generate 10 select data items without optgroups:**
```twig
{{ tp_select_data() }}
```
_Output:_
```twig
[
  {"type": "option", "value": "item_1", "label": "Lorem ipsum", "selected": false},
  {"type": "option", "value": "item_2", "label": "Dolor sit", "selected": false},
  {"type": "option", "value": "item_3", "label": "Amet consectetur", "selected": false},
  {"type": "option", "value": "item_1", "label": "Adipiscing elit", "selected": false},
  {"type": "option", "value": "item_2", "label": "Nullam suscipit", "selected": false},
  {"type": "option", "value": "item_3", "label": "Vitae felis", "selected": false},
  {"type": "option", "value": "item_1", "label": "Et convallis", "selected": false},
  {"type": "option", "value": "item_2", "label": "Nibh vitae", "selected": false},
  {"type": "option", "value": "item_3", "label": "Eros tempor", "selected": false},
  {"type": "option", "value": "item_3", "label": "Tempus sit", "selected": false}
]
```

**Generate 4 select data items across 2 optgroups:**
```twig
{{ tp_select_data(4, 2) }}
```
_Output:_
```twig
[
  {"type": "optgroup", "label": "Nullam suscipit", "options": [
    {"type": "option", "value": "item_1", "label": "Lorem ipsum", "selected": false},
    {"type": "option", "value": "item_2", "label": "Dolor sit", "selected": false}
  ]},
  {"type": "optgroup", "label": "Vitae felis", "options": [
    {"type": "option", "value": "item_3", "label": "Amet consectetur", "selected": false},
    {"type": "option", "value": "item_4", "label": "Adipiscing elit", "selected": false}
  ]}
]
```

**Generate 6 select data items across 2 optgroups, distributed randomly:**
```twig
{{ tp_select_data(6, 2, true) }}
```
_Output:_
```twig
[
  {"type": "optgroup", "label": "Nullam suscipit", "options": [
    {"type": "option", "value": "item_5", "label": "Nibh vitae", "selected": false},
    {"type": "option", "value": "item_2", "label": "Et convallis", "selected": false},
    {"type": "option", "value": "item_6", "label": "Lorem ipsum", "selected": false},
    {"type": "option", "value": "item_1", "label": "Dolor sit", "selected": false}
  ]},
  {"type": "optgroup", "label": "Vitae felis", "options": [
    {"type": "option", "value": "item_4", "label": "Amet consectetur", "selected": false},
    {"type": "option", "value": "item_3", "label": "Adipiscing elit", "selected": false}
  ]}
]
```

### Generating Placeholder Videos
The `tp_video` function generates a placeholder video URL or an entire `<video>` tag with customisable attributes.

```twig
{{ tp_video() }}
```

#### **Optional Arguments:**
| Parameter           | Type   | Default | Description |
|---------------------|--------|---------|-------------|
| `url_only`          | bool   | `false` | If `true`, returns only the video URL instead of a `<video>` tag. |
| `video_id`          | string | `null`  | Video ID from ['Big_Buck_Bunny', 'Jellyfish', 'Sintel']. Defaults to a random selection. |
| `video_size`        | string | `1080`  | Video resolution size from ['1080', '720', '360']. |
| `video_extension`   | string | `mp4`   | Video file extension from ['mp4', 'webm', 'mkv']. |
| `video_autoplay`    | bool   | `false` | If `true`, autoplay the video. Only works if `url_only` is set to `false`. |
| `video_controls`    | bool   | `true`  | If `true`, show video controls. Only works if `url_only` is set to `false`. |
| `video_loop`        | bool   | `false` | If `true`, loop the video. Only works if `url_only` is set to `false`. |
| `video_muted`       | bool   | `false` | If `true`, mute the video. Only works if `url_only` is set to `false`. |
| `video_playsinline` | bool   | `true`  | If `true`, play the video inline. Only works if `url_only` is set to `false`. |

#### **Examples:**

**Generate a default placeholder video:**
```twig
{{ tp_video() }}
```
_Output:_
```html
<video controls playsinline width="1920" height="1080">
  <source src="https://test-videos.co.uk/vids/bigbuckbunny/mp4/h264/1080/Big_Buck_Bunny_10s_1MB.mp4" type="video/mp4" />
  Your browser does not support HTML5 video. <a href="https://test-videos.co.uk/vids/bigbuckbunny/mp4/h264/1080/Big_Buck_Bunny_10s_1MB.mp4">Download the video</a> instead.
</video>
```

**Generate only the video URL:**
```twig
{{ tp_video(true) }}
```
_Output:_
```text
https://test-videos.co.uk/vids/jellyfish/mp4/h264/1080/Jellyfish_10s_1MB.mp4
```

**Generate a random video with autoplay, no controls, loop, muted, and playsinline to use as a background video:**
```twig
{{ tp_video(false, null, null, null, true, false, true, true, true) }}
```
_Output:_
```html
<video autoplay loop muted playsinline width="1920" height="1080">
  <source src="https://test-videos.co.uk/vids/sintel/mp4/h264/1080/Sintel_10s_1MB.mp4" type="video/mp4" />
  Your browser does not support HTML5 video. <a href="https://test-videos.co.uk/vids/sintel/mp4/h264/1080/Sintel_10s_1MB.mp4">Download the video</a> instead.
</video>
```

**Generate a URL for the jellyfish video in 360p with mkv extension:**
```twig
{{ tp_video(true, 'Jellyfish', '360', 'mkv') }}
```
_Output:_
```text
https://test-videos.co.uk/vids/jellyfish/mkv/360/Jellyfish_10s_1MB.mkv
```

## Requirements

- Drupal **10.3+**
- A custom or existing theme to use Twig placeholder functionality.

## Installation

Install via Composer:
```bash
composer require drupal/twig_placeholders
```

Enable the module using Drush or the Drupal UI:
```bash
drush en twig_placeholders
```

## Support & Contributions

If you encounter any issues or have feature requests, please submit them on the [Twig Placeholders issue queue](https://www.drupal.org/project/twig_placeholders/issues).

Contributions are welcome! Feel free to submit a pull request or suggest improvements.

---

Enjoy using **Twig Placeholders** to demo all of your best Drupal components! 😎
